#include "Engine/PhongLighting.es30.glsl"

layout(location=0) out vec4 target0;
layout(location=1) out uvec4 target1;

void main()
{
  VertexIn In = importIn();
  SurfaceOut surface = SURFACE_FUNCTION(In);

#if defined(COGS_ALPHA_TEST_DISCARD) || defined(COGS_ALPHA_TO_COVERAGE)
  if (surface.Color.a < alphaThreshold) discard;
#endif

  if ((GetMaterialId() & COGS_LIGHT_MASK_PHONG) == COGS_LIGHT_MASK_PHONG) {

    float light = 0.50;
    for (int i = 0; i < COGS_MAX_LIGHTS; i++) {
      vec4 lightPosition = lightPositions[i];

      float shadow = 1.0;
#ifdef COGS_FORWARD_SHADOWS_ENABLED
      if ((GetMaterialId() & COGS_LIGHT_MASK_SHADOW_RECEIVER) == COGS_LIGHT_MASK_SHADOW_RECEIVER) {
        if(lightPositions[i].w == 0.0) {
#if COGS_NUM_DIRECTIONAL_SHADOWS > 0
          shadow = calculateShadowContribution(In.WorldPosition, surface.Normal, lightDirections[i].xyz, i, vec4(0.0));
#endif
      }
      else {
#if COGS_NUM_POINT_SHADOWS > 0
          shadow = calculatePointLightShadow(lightPosition, In.WorldPosition.xyz, surface.Normal, i);
#endif
        }
      }
#endif
      vec3 offset = In.WorldPosition.xyz - lightPosition.xyz;
      vec3 lightDir = lightDirections[i].w == 0.f ? lightDirections[i].xyz : normalize(offset);

      light +=  0.5 * smoothstep(0.0, 0.05, shadow * dot(surface.Normal, -lightDir));
    }
    surface.Color.rgb *= light;

    vec3 n = 512.0 * clamp(vec3(1.0) + mat3(viewMatrix) * surface.Normal, vec3(0.0), vec3(2.0));
    target1 = uvec4(n.x, n.y, n.z, 0);
  }
  else {
    target1 = uvec4(512, 512, 512, 0);
  }
  target0 = surface.Color;
}
