Texture2D colorTexture;
Texture2D<uint4> normalTexture;
Texture2D depthTexture;

cbuffer Params {

  float4x4 inverseProjection;
  float2 viewSizeRcp;
};

float3 worldSpaceFromScreen(float2 screenPos, float depth)
{
  float2 unit = viewSizeRcp * screenPos;
  unit.y = 1.0 - unit.y;

  float4 h = mul(float4(2.0 * unit - (float2)1.0, depth, 1.0), inverseProjection);
  return (1.0 / h.w) * h.xyz;
}

float4 main(float4 ScreenPos : SV_Position) : SV_Target
{
  const float3 n00 = (1.0 / 512.0) * normalTexture.Load(int3(ScreenPos.xy, 0)).rgb - (float3)1.0;
  const float3 n10 = (1.0 / 512.0) * normalTexture.Load(int3(ScreenPos.xy, 0) + int3(1, 0, 0)).rgb - (float3)1.0;
  const float3 n01 = (1.0 / 512.0) * normalTexture.Load(int3(ScreenPos.xy, 0) + int3(0, 1, 0)).rgb - (float3)1.0;

  float intensity = 1.0;
  if (0.1 < dot(n00, n00) || 0.1 < dot(n10, n10) || 0.1 < dot(n01, n01))
  {
    const float d_00 = depthTexture.Load(int3(ScreenPos.xy, 0) + int3(0, 0, 0)).r;
    const float d_10 = depthTexture.Load(int3(ScreenPos.xy, 0) + int3(1, 0, 0)).r;
    const float d_01 = depthTexture.Load(int3(ScreenPos.xy, 0) + int3(0, 1, 0)).r;

    const float3 wpos_00 = worldSpaceFromScreen(ScreenPos.xy + float2(0, 0), d_00);
    const float3 wpos_10 = worldSpaceFromScreen(ScreenPos.xy + float2(1, 0), d_10);
    const float3 wpos_01 = worldSpaceFromScreen(ScreenPos.xy + float2(0, 1), d_01);

    // Distance from tangent plane
    float3 dir_10 = normalize(wpos_10 - wpos_00);
    float3 dir_01 = normalize(wpos_01 - wpos_00);
    float g = max(max(dot(n00, dir_01),
                      dot(n01, dir_01)),
                  max(dot(n00, dir_10),
                      dot(n10, dir_10)));

    // Difference in normals
    float a = min(dot(n00, n10),
                  dot(n00, n01));

    if (0.3 < g || a < 0.95) {
      intensity = 0.25;
    }
  }

  const float4 color = colorTexture.Load(int3(ScreenPos.xy, 0));
  return float4(intensity * color.rgb,
                color.a);
}
