#include "Engine/PhongLighting.hlsl"

SurfaceOut surfaceFunction(VertexIn In)
{
  SurfaceOut Out = (SurfaceOut)0;

#ifdef COGS_BASIC_SKY_LOD
  const int lod = int(COGS_BASIC_SKY_LOD);
#else
  const int lod = 0;
#endif

#if defined(COGS_BASIC_SKY_SOURCE_DIFFUSE)
  float4 textureColor = diffuseMap.SampleLevel(diffuseMapSampler, In.TexCoord, lod);
#elif defined(COGS_BASIC_SKY_SOURCE_CUBE)
  float4 textureColor = cubeMap.SampleLevel(cubeMapSampler, In.Direction.xzy, lod);
#elif defined(COGS_BASIC_SKY_SOURCE_ENV_SKY)
  float4 textureColor = environmentSky.SampleLevel(environmentSkySampler, In.Direction.xzy, lod);
  textureColor.rgb *= environmentBrightness;
#elif defined(COGS_BASIC_SKY_SOURCE_ENV_RADIANCE)
  float4 textureColor = environmentRadiance.SampleLevel(environmentRadianceSampler, In.Direction.xzy, 0);
  textureColor.rgb *= environmentBrightness;
#elif defined(COGS_BASIC_SKY_SOURCE_ENV_IRRADIANCE)
  float4 textureColor = environmentIrradiance.SampleLevel(environmentIrradianceSampler, In.Direction.xzy, lod);
  textureColor.rgb *= environmentBrightness;
#else
  float4 textureColor = float4(1, 1, 1, 1);
#endif
  textureColor.rgb *= skyMultiplier;

  float3 color = lerp(diffuseColor, textureColor * diffuseColor, textureColor.a).rgb * brightness;

  Out.Color = float4(color, diffuseColor.a * textureColor.a);
  Out.Normal = float3(1, 1, 1);

  return Out;
}
