#include "Engine/Common.hlsl"
#include "Engine/TransparencyCommon.hlsl"
#include "Engine/PhongLighting.hlsl"

RWByteAddressBuffer               List;     // UAV, size = width x height, initialized to ~0.
RWStructuredBuffer<FragmentNode>  Node;     // No need for UAV, size = alloc
RWStructuredBuffer<FragmentData>  Data;     // No need for UAV, size = alloc

[earlydepthstencil]
void main(VertexIn In, float4 ScreenPos : SV_Position, bool facing : SV_IsFrontFace)
{
  SurfaceOut surface = surfaceFunction(In);

#ifdef COGS_ALPHA_TEST_DISCARD
  if (surface.Color.a <= alphaThreshold) discard;
#endif

  // Get fragment index
  uint nodeIndex = Node.IncrementCounter();

  // If we have more room, insert node into list
  if (nodeIndex < NodeAlloc) {
    uint startOffset = 4 * ((uint)ScreenPos.y * Width + (uint)ScreenPos.x);

    // Insert fragment at front of pixel location's list and retrieve the previously stored fragment
    // index.
    uint nextNodeIndex;
    uint nodeIndexWithBits = nodeIndex | (facing ? 0x80000000u : 0u);
    List.InterlockedExchange(startOffset, nodeIndexWithBits, nextNodeIndex);

    Node[nodeIndex].Depth = ScreenPos.z;
    Node[nodeIndex].Next = nextNodeIndex ;

    pack(Data[nodeIndex],
         ScreenPos,
         surface.Normal,
         GetMaterialId(),
         surface.Color.a,
         surface.Emissive,
         surface.Color.rgb,
         surface.Specular,
         surface.SpecularPower);
  }
}
