cbuffer TemporalParameters
{
  float4x4 inverseClipToWorldMatrix;
  float4x4 prevWorldToClipMatrix;
  uint2 scale;
  uint2 off;
  uint2 size;
};

struct VertexIn
{
  float4 Position : SV_Position;
  float4 ScreenPosition : TEXCOORD0;
  float2 TexCoords : TEXCOORD1;
};

struct PixelOut
{
  float4 Color : SV_Target;
};

Texture2D depthTexture;
Texture2D oitTexture;
Texture2D historyTexture;
SamplerState linearSampler;
SamplerState historySampler;

PixelOut main(VertexIn In, float4 ScreenPos : SV_Position)
{
  const float depth = depthTexture.Load(int3(ScreenPos.xy, 0)).r;

  uint2 sp = ScreenPos.xy;
  uint2 oitoff = sp%scale;
  uint2 oitpos = sp/scale;

  float4 oit;
  float4 oit_max = float4(-9999, -9999, -9999, -9999);
  float4 oit_min = float4(9999, 9999, 9999, 9999);
  for(int i=-1; i<=1; i++){
    for(int j=-1; j<=1; j++){
      float4 tmp = oitTexture.Load(int3(oitpos+int2(i, j), 0));
      oit_max = max(tmp, oit_max);
      oit_min = min(tmp, oit_min);
      if(i==0 && j==0) oit = tmp;
    }
  }

  float4 worldPosition = mul(float4(In.ScreenPosition.xy, depth, 1), inverseClipToWorldMatrix);
  worldPosition = float4(worldPosition.xyz/worldPosition.w, 1.0);
  float4 clip = mul(worldPosition, prevWorldToClipMatrix);
  float3 ndc = clip.xyz/clip.w;
  float2 prevScreenPos = (ndc.xy*float2(0.5, -0.5)+float2(0.5, 0.5));
  float2 motionVector = prevScreenPos - ScreenPos.xy/size;

  // float4 history = historyTexture.Load(int3(ScreenPos.xy, 0));
  float4 history = historyTexture.SampleLevel(historySampler, prevScreenPos, 0);

  if(any(saturate(prevScreenPos) != prevScreenPos))
    history = oit;
  else{
    float max_motion = max(abs(motionVector.x)*size.x, abs(motionVector.y)*size.y);
    if(max_motion > 1.0)
      history = clamp(history, oit_min, oit_max);
  }

  PixelOut Out;
  if(oitoff.x == off.x && oitoff.y == off.y)
    Out.Color = oit;
  else
    Out.Color = history;
  // if(depth != 1.0){
  //   Out.Color = float4(abs(motionVector*size), depth, 1.0);
  // }
  return Out;
}
