#include "Engine/Common.hlsl"

Texture2D depthTexture;
SamplerState linearClampSampler;

struct VertexIn
{
  float4 Position : SV_Position;
  float4 ScreenPosition : TEXCOORD0;
  float2 TexCoords : TEXCOORD1;
};

float ndcToViewDepth(float ndcDepth)
{
#ifdef COGS_REVERSE_DEPTH
  return -projectionMatrix[2][3] / (ndcDepth - projectionMatrix[2][2]);
#else
  return -projectionMatrix[2][3] / (ndcDepth + projectionMatrix[2][2]);
#endif
}

float4 main(VertexIn In) : SV_Target
{
  float2 pixelSize;
  depthTexture.GetDimensions(pixelSize.x, pixelSize.y);

  float2 texCoords = In.TexCoords + float2(-0.25, -0.25) * (1.0 / pixelSize);
  float depthNdc = depthTexture.Sample(linearClampSampler, texCoords).x;

#ifdef COGS_REVERSE_DEPTH
  depthNdc = -depthNdc;
#endif

  float depth = ndcToViewDepth(depthNdc);

  return depth;
}
