#include "Engine/Common.hlsl"

Texture2D depthTexture;
Texture2D depth4xTexture;
Texture2D occlusion4xTexture;

SamplerState linearClampSampler;

struct VertexIn
{
  float4 Position : SV_Position;
  float4 ScreenPosition : TEXCOORD0;
  float2 TexCoords : TEXCOORD1;
};

float4 main(VertexIn In) : SV_Target
{
  float2 pixelSize;
  depthTexture.GetDimensions(pixelSize.x, pixelSize.y);

  float depth = depthTexture.Sample(linearClampSampler, In.TexCoords).x;
#ifdef COGS_REVERSE_DEPTH
  depth = -depth;
#endif

  float4 inDepth = depth4xTexture.GatherRed(linearClampSampler, In.TexCoords).wzxy;
  float4 depthsDiffs = abs(depth.xxxx - inDepth);

  float4 inSsao = occlusion4xTexture.GatherRed(linearClampSampler, In.TexCoords).wzxy;

  float2 imageCoord = In.TexCoords / (1.0 / pixelSize);
  float2 fractional = frac(imageCoord);

  float a = (1.0f - fractional.x) * (1.0f - fractional.y);
  float b = fractional.x * (1.0f - fractional.y);
  float c = (1.0f - fractional.x) * fractional.y;
  float d = fractional.x * fractional.y;

  float4 ssao = 0.0f;
  float weightsSum = 0.0f;

  float depthOffset = 0.001f;

  float weight00 = a / (depthsDiffs.x + depthOffset);
  ssao += weight00 * inSsao.x;
  weightsSum += weight00;

  float weight10 = b / (depthsDiffs.y + depthOffset);
  ssao += weight10 * inSsao.y;
  weightsSum += weight10;

  float weight01 = c / (depthsDiffs.z + depthOffset);
  ssao += weight01 * inSsao.z;
  weightsSum += weight01;

  float weight11 = d / (depthsDiffs.w + depthOffset);
  ssao += weight11 * inSsao.w;
  weightsSum += weight11;

  ssao /= weightsSum;

  return ssao;
}
