// uint3 : SV_GroupThreadID     Thread id within block
// uint3 : SV_GroupID           Block id
// uint3 : SV_DispatchThreadID  Global thread id
// uint  : SV_GroupIndex        Linearized thread id within block

// groupshared shmem[64];  Declaration of shared memory

//AllMemoryBarrier - block execution until all memory accesses has completed. (my writes visible to world?)
//AllMemoryBarrierWithGroupSync - Above + wait until threads in group have reached this call.
//DeviceMemoryBarrier
//DeviceMemoryBarrierWithGroupSync
//GroupMemoryBarrier
//GroupMemoryBarrierWithGroupSync();

float lightScale;
float expComp;
float expMin;
float expMax;
float expIncFactor;
float expDecFactor;

RWStructuredBuffer<float> exposureBuffer;
RWStructuredBuffer<float> scratchBuffer;

groupshared float reduce[256];

[numthreads(256,1,1)]
void main(uint3 gid : SV_GroupID, uint3 tid : SV_GroupThreadID, uint tix : SV_GroupIndex)
{
  // 16x16=256 threads.
  uint tx = tid.x & 0xfu;
  uint ty = (tid.x >> 4);

  reduce[tid.x] = scratchBuffer[tid.x];

  GroupMemoryBarrierWithGroupSync();

  if (ty == 0) {
    float avg = 0.0;
    for (int i = 0; i < 16; i++) {
      avg += reduce[16 * i + tx];
    }
    reduce[tx] = avg;
  }

  GroupMemoryBarrierWithGroupSync();

  if ((ty == 0) && (tx == 0)) {
    float avg = 0.0;
    for (int i = 0; i < 16; i++) {
      avg += reduce[i];
    }
    avg = lightScale * exp(avg);

    float oldExposure = exposureBuffer[0];
    float newExposure = clamp(exp2(expComp - 1) / avg,
                              expMin,
                              expMax);

    float exposure;
    
    if (!isfinite(oldExposure) || (oldExposure < expMin) || (expMax < oldExposure)) {
      exposure = newExposure;
    } else if (newExposure < oldExposure) {
      exposure = max(newExposure, expDecFactor * oldExposure);
    } else {
      exposure = min(newExposure, expIncFactor * oldExposure);
    }

    exposureBuffer[0] = exposure;
  }
}
