#ifdef COGS_REVERSE_DEPTH
#define COGS_CLEAR_DEPTH 0.0
#else
#define COGS_CLEAR_DEPTH 1.0
#endif

vec2 TransformImage(in vec2 offset, in float aspect)
{
  vec4 pos = vec4(offset.x * aspect, offset.y, 0, 1);

  pos = pos * imageTransform();

  return vec2(pos.x / aspect, pos.y);
}

void quadExpand(out vec4 position,
                out vec4 weights,
                out vec2 texCoords,
                in vec4 inPosition,
                in vec4 inParameters,
                in vec4 inTexCoords,
                in float index)
{
  vec3 p = inPosition.xyz / inPosition.w;
  vec2 size = inParameters.xy;
  vec2 offset = inParameters.zw;
  vec2 viewportScale = 1.0 / (0.5 * viewport.zw);
  vec2 scaledSize = size * viewportScale;
  vec2 center = viewport.zw * (0.5 + 0.5 * p.xy);
  float aspect = viewport.z / viewport.w;
  float depth = alwaysOnTop() ? (1.0 - COGS_CLEAR_DEPTH) : p.z;
  vec2 scaledCenter = (center + offset) * viewportScale;
  vec4 mask = vec4(index == 0.0,
                   index == 1.0,
                   index == 2.0,
                   index == 3.0);
  texCoords = vec2(dot(mask, inTexCoords.xzxz),
                     dot(mask, inTexCoords.yyww));
  weights = vec4(dot(mask, vec4(0, 1, 0, 1)),
                   dot(mask, vec4(0, 0, 1, 1)),
                   0,
                   0);

#ifdef COGS_SPRITES_NOALIGNPIXEL
  vec2 ll = center + offset + vec2(-0.5, -0.5) * size;
#else
  vec2 ll = floor(center + offset + vec2(-0.5, -0.5) * size);
#endif

#ifdef COGS_SPRITES_IMAGETRANSFORM
  vec2 v = scaledCenter + TransformImage((weights.xy-vec2(0.5)) * scaledSize, aspect) - vec2(1, 1);
#else
  vec2 v = (ll + weights.xy * size) * viewportScale - vec2(1, 1);
#endif
  position = vec4(v, depth, 1.0);
}
