vec3 GetNormal(in VertexIn In)
{
#ifdef COGS_STANDARD_MATERIAL_NORMAL_MAP
  vec3 normal = normalize(In.Normal);
  vec3 tangent = normalize(In.Tangent);

#if COGS_STANDARD_MATERIAL_NORMAL_UV == 0
  vec3 normalColor = 2.0 * texture2D(normalMap, In.TexCoord0).rgb - vec3(1.0);
#elif COGS_STANDARD_MATERIAL_NORMAL_UV == 1
  vec3 normalColor = 2.0 * texture2D(normalMap, In.TexCoord1).rgb - vec3(1.0);
#endif

#ifdef COGS_STANDARD_MATERIAL_HAS_BITANGENT
  vec3 bitangent = normalize(In.Bitangent);
#else
  vec3 bitangent = normalize(cross(normal, tangent));
#endif

  vec3 bumpNormal = normalize(normalColor.z * normal + normalColor.x * tangent + normalColor.y * bitangent);
  normal = normalize(mix(normal, bumpNormal, normalMapFactor()));

  return In.IsFrontFace ? normal : -normal;
#else
  return normalize(In.IsFrontFace ? In.Normal : -In.Normal);
#endif
}

vec4 GetAlbedo(in VertexIn In)
{
#ifdef COGS_STANDARD_MATERIAL_ALBEDO_MAP

#if COGS_STANDARD_MATERIAL_ALBEDO_UV == 0
  return albedo() * texture2D(albedoMap, In.TexCoord0.xy);
#elif COGS_STANDARD_MATERIAL_ALBEDO_UV == 1
  return albedo() * texture2D(albedoMap, In.TexCoord1.xy);
#endif

#else
  return albedo();
#endif
}

float GetRoughness(in VertexIn In)
{
#ifdef COGS_STANDARD_MATERIAL_ROUGHNESS_MAP

#if COGS_STANDARD_MATERIAL_ROUGHNESS_UV == 0
  return roughness() * texture2D(roughnessMap, In.TexCoord0).r;
#elif COGS_STANDARD_MATERIAL_ROUGHNESS_UV == 1
  return roughness() * texture2D(roughnessMap, In.TexCoord1).r;
#endif

#elif defined(COGS_STANDARD_MATERIAL_METALLIC_ROUGHNESS_MAP)

#if COGS_STANDARD_MATERIAL_METALLIC_ROUGHNESS_UV == 0
  return roughness() * texture2D(metallicRoughnessMap, In.TexCoord0).g;
#elif COGS_STANDARD_MATERIAL_METALLIC_ROUGHNESS_UV == 1
  return roughness() * texture2D(metallicRoughnessMap, In.TexCoord1).g;
#endif

#else
  return roughness();
#endif
}

float GetMetallic(in VertexIn In)
{
#ifdef COGS_STANDARD_MATERIAL_METALLIC_MAP

#if COGS_STANDARD_MATERIAL_METALLIC_UV == 0
  return metallic() * texture2D(metallicMap, In.TexCoord0).r;
#elif COGS_STANDARD_MATERIAL_METALLIC_UV == 1
  return metallic() * texture2D(metallicMap, In.TexCoord1).r;
#endif

#elif defined(COGS_STANDARD_MATERIAL_METALLIC_ROUGHNESS_MAP)

#if COGS_STANDARD_MATERIAL_METALLIC_ROUGHNESS_UV == 0
  return metallic() * texture2D(metallicRoughnessMap, In.TexCoord0).b;
#elif COGS_STANDARD_MATERIAL_METALLIC_ROUGHNESS_UV == 1
  return metallic() * texture2D(metallicRoughnessMap, In.TexCoord1).b;
#endif

#else
  return metallic();
#endif
}

float GetGloss(in VertexIn In)
{
#ifdef COGS_STANDARD_MATERIAL_GLOSS_MAP

#if COGS_STANDARD_MATERIAL_GLOSS_UV == 0
  return gloss() * texture2D(glossMap, In.TexCoord0).r;
#elif COGS_STANDARD_MATERIAL_GLOSS_UV == 1
  return gloss() * texture2D(glossMap, In.TexCoord1).r;
#endif

#elif defined(COGS_STANDARD_MATERIAL_SPECULAR_GLOSS_MAP)

#if COGS_STANDARD_MATERIAL_SPECULAR_GLOSS_UV == 0
  return gloss() * texture2D(specularGlossMap, In.TexCoord0).a;
#elif COGS_STANDARD_MATERIAL_SPECULAR_GLOSS_UV == 1
  return gloss() * texture2D(specularGlossMap, In.TexCoord1).a;
#endif

#else
  return gloss();
#endif
}

vec3 GetSpecular(in VertexIn In)
{
#ifdef COGS_STANDARD_MATERIAL_SPECULAR_MAP

#if COGS_STANDARD_MATERIAL_SPECULAR_UV == 0
  return specular() * texture2D(specularMap, In.TexCoord0).rgb;
#elif COGS_STANDARD_MATERIAL_SPECULAR_UV == 1
  return specular() * texture2D(specularMap, In.TexCoord1).rgb;
#endif

#elif defined(COGS_STANDARD_MATERIAL_SPECULAR_GLOSS_MAP)

#if COGS_STANDARD_MATERIAL_SPECULAR_GLOSS_UV == 0
  return specular() * texture2D(specularGlossMap, In.TexCoord0).rgb;
#elif COGS_STANDARD_MATERIAL_SPECULAR_GLOSS_UV == 1
  return specular() * texture2D(specularGlossMap, In.TexCoord1).rgb;
#endif

#else
  return specular();
#endif
}

vec3 GetEmissive(in VertexIn In)
{
#ifdef COGS_STANDARD_MATERIAL_EMISSIVE_MAP

#if COGS_STANDARD_MATERIAL_EMISSIVE_UV == 0
  return emissive() * texture2D(emissiveMap, In.TexCoord0.xy).rgb;
#elif COGS_STANDARD_MATERIAL_EMISSIVE_UV == 1
  return emissive() * texture2D(emissiveMap, In.TexCoord1.xy).rgb;
#endif

#else
  return emissive();
#endif
}

float GetOcclusion(in VertexIn In)
{
#ifdef COGS_STANDARD_MATERIAL_OCCLUSION_MAP

#if COGS_STANDARD_MATERIAL_OCCLUSION_UV == 0
  return occlusion() * texture2D(occlusionMap, In.TexCoord0).r;
#elif COGS_STANDARD_MATERIAL_OCCLUSION_UV == 1
  return occlusion() * texture2D(occlusionMap, In.TexCoord1).r;
#endif

#else
  return occlusion();
#endif
}
