VertexOut vertexFunction(VertexIn In)
{
  VertexOut Out = (VertexOut)0;

#ifdef COGS_VARIANT_SKINNED_BASIC
  const float w0 = In.BoneWeights.x;
  const int i0 = In.BoneIndexes.x;

  const float w1 = In.BoneWeights.y;
  const int i1 = In.BoneIndexes.y;

  const float w2 = In.BoneWeights.z;
  const int i2 = In.BoneIndexes.z;

  const float w3 = In.BoneWeights.w;
  const int i3 = In.BoneIndexes.w;

  float4 position = w0 * mul(In.Position, boneTransforms[i0]);
  position += w1 * mul(In.Position, boneTransforms[i1]);
  position += w2 * mul(In.Position, boneTransforms[i2]);
  position += w3 * mul(In.Position, boneTransforms[i3]);

  float3 normal = w0 * mul(In.Normal, (float3x3)boneTransforms[i0]);
  normal += w1 * mul(In.Normal, (float3x3)boneTransforms[i1]);
  normal += w2 * mul(In.Normal, (float3x3)boneTransforms[i2]);
  normal += w3 * mul(In.Normal, (float3x3)boneTransforms[i3]);

#ifdef COGS_STANDARD_MATERIAL_NORMAL_MAP
  float3 tangent = w0 * mul(In.Tangent.xyz, (float3x3)boneTransforms[i0]);

  tangent += w1 * mul(In.Tangent.xyz, (float3x3)boneTransforms[i1]);
  tangent += w2 * mul(In.Tangent.xyz, (float3x3)boneTransforms[i2]);
  tangent += w3 * mul(In.Tangent.xyz, (float3x3)boneTransforms[i3]);

#if BASE_MATERIAL_VERTEX_TANGENT0 == COGS_FLOAT4
  float tangentW = In.Tangent.w;
#else
  float tangentW = 1.0;
#endif

#ifdef COGS_STANDARD_MATERIAL_HAS_BITANGENT
  float3 bitangent = w0 * mul(In.Bitangent, (float3x3)boneTransforms[i0]);
  bitangent += w1 * mul(In.Bitangent, (float3x3)boneTransforms[i1]);
  bitangent += w2 * mul(In.Bitangent, (float3x3)boneTransforms[i2]);
  bitangent += w3 * mul(In.Bitangent, (float3x3)boneTransforms[i3]);
#endif
#endif
#else
  float4 position = In.Position;
  float3 normal = In.Normal;
#ifdef COGS_STANDARD_MATERIAL_NORMAL_MAP
  float3 tangent = In.Tangent.xyz;

#ifdef COGS_STANDARD_MATERIAL_HAS_BITANGENT
  float3 bitangent = In.Bitangent;
#endif
#endif
#endif

  Out.WorldPosition = mul(position, worldMatrix);
  Out.Position = mul(Out.WorldPosition, worldToClipMatrix);

  Out.Normal = mul(normal, (float3x3)worldMatrix);
#ifdef COGS_STANDARD_MATERIAL_NORMAL_MAP
  Out.Tangent = mul(tangent, (float3x3)worldMatrix);
#ifdef COGS_STANDARD_MATERIAL_HAS_BITANGENT
  Out.Bitangent = mul(bitangent, (float3x3)worldMatrix);
#endif
#endif

#ifdef COGS_STANDARD_MATERIAL_TEXTURED
  Out.TexCoord0 = In.TexCoord0;
#endif

#ifdef COGS_STANDARD_MATERIAL_SECONDARY_UV
  Out.TexCoord1 = In.TexCoord1;
#endif

#ifdef COGS_STANDARD_MATERIAL_ALBEDO_PER_VERTEX
  Out.Albedo = In.Albedo;
#endif

  return Out;
}
